@@ -56,6 +56,8 @@
 #' @param nr.adapt            Number of iterations in the adaptation process, default is 1000. Some models may need more iterations during adptation.
 #' @param nr.burnin           Number of iteration discard for burn-in period, default is 1000. Some models may need a longer burnin period.
 #' @param nr.thin             Thinning rate, it must be a positive integer, the default value 1.
+#' @param parallel            NULL -> jags, 'jags.parallel' -> jags.parallel execution
+#'
 #' @return                    This function returns an object of the class "bcmixmeta". This object contains the MCMC
 #'                            output of each parameter and hyper-parameter in the model and
 #'                            the data frame used for fitting the model.
@@ -195,7 +197,8 @@ bcmixmeta = function(
     nr.iterations   = 10000,
     nr.adapt        = 1000,
     nr.burnin       = 1000,
-    nr.thin         = 1)UseMethod("bcmixmeta")
+    nr.thin         = 1,
+    parallel        = NULL)UseMethod("bcmixmeta")
 
 
 #' @export
@@ -222,10 +225,11 @@ bcmixmeta.default = function(
     nr.iterations   = 10000,
     nr.adapt        = 1000,
     nr.burnin       = 1000,
-    nr.thin         = 1
+    nr.thin         = 1,
+    parallel        = NULL
 )
 {
-
+  if(!is.null(parallel) && parallel != "jags.parallel") stop("The parallel option must be NULL or 'jags.parallel'")
 
   # Data sort to facilitate the mixture setup
   # y = sort(data$TE)
@@ -262,51 +266,66 @@ bcmixmeta.default = function(
   }
 
   # This list describes the data used by the BUGS script.
-  data.bcmixmeta.delta = list ("y", "se.y", "N", "T",
-                               "mean.mu.0",
-                               "sd.mu.0",
-                               "scale.sigma.between",
-                               "df.scale.between",
-                               "scale.sigma.beta",
-                               "df.scale.beta",
-                               "B.lower",
-                               "B.upper",
-                               "a.0",
-                               "a.1",
-                               "alpha.0",
-                               "alpha.1",
-                               "K")
-
-  data.bcmixmeta.x = list ("y", "se.y", "x",
-                           "N", "T",
-                           "mean.mu.0",
-                           "sd.mu.0",
-                           "scale.sigma.between",
-                           "df.scale.between",
-                           "scale.sigma.beta",
-                           "df.scale.beta",
-                           "B.lower",
-                           "B.upper",
-                           "a.0",
-                           "a.1",
-                           "alpha.0",
-                           "alpha.1",
-                           "K")
+  data.bcmixmeta.delta <-
+    list(y = y,
+         se.y = se.y,
+         N = N,
+         T = T,
+         mean.mu.0 = mean.mu.0,
+         sd.mu.0 = sd.mu.0,
+         scale.sigma.between = scale.sigma.between,
+         df.scale.between = df.scale.between,
+         scale.sigma.beta = scale.sigma.beta,
+         df.scale.beta = df.scale.beta,
+         B.lower = B.lower,
+         B.upper = B.upper,
+         a.0 = a.0,
+         a.1 = a.1,
+         alpha.0 = alpha.0,
+         alpha.1 = alpha.1,
+         K = K
+    )
 
+  data.bcmixmeta.x <-
+    list(y = y,
+         se.y = se.y,
+         x = x,
+         N = N,
+         T = T,
+         mean.mu.0 = mean.mu.0,
+         sd.mu.0 = sd.mu.0,
+         scale.sigma.between = scale.sigma.between,
+         df.scale.between = df.scale.between,
+         scale.sigma.beta = scale.sigma.beta,
+         df.scale.beta = df.scale.beta,
+         B.lower = B.lower,
+         B.upper = B.upper,
+         a.0 = a.0,
+         a.1 = a.1,
+         alpha.0 = alpha.0,
+         alpha.1 = alpha.1,
+         K = K
+    )
 
   # This list describes the data used by the BUGS script.
-  data.bcmixmeta.bilateral.bias = list ("y", "se.y", "N", "T",
-                              "mean.mu.0",
-                              "sd.mu.0",
-                              "scale.sigma.between",
-                              "df.scale.between",
-                              "B.lower",
-                              "B.upper",
-                              "a.0",
-                              "a.1",
-                              "alpha.0",
-                              "alpha.1",
-                              "K")
+  data.bcmixmeta.bilateral.bias <-
+    list(y = y,
+         se.y = se.y,
+         N = N,
+         T = T,
+         mean.mu.0 = mean.mu.0,
+         sd.mu.0 = sd.mu.0,
+         scale.sigma.between = scale.sigma.between,
+         df.scale.between = df.scale.between,
+         B.lower = B.lower,
+         B.upper = B.upper,
+         a.0 = a.0,
+         a.1 = a.1,
+         alpha.0 = alpha.0,
+         alpha.1 = alpha.1,
+         K = K
+    )
+
 
 
   if(bilateral.bias == TRUE)
@@ -725,28 +744,58 @@ mu.new ~ dnorm(mu.0,  inv.var.0)
   "
 
 
-  if(bilateral.bias == TRUE)
-  {model.bugs.connection = textConnection(model.bugs.bcmixmeta.bilateral.bias)}
-  else
-  {model.bugs.connection = textConnection(model.bugs.bcmixmeta.delta)}
+  if(bilateral.bias == TRUE){
+    model.bugs.connection = textConnection(model.bugs.bcmixmeta.bilateral.bias)
+    model.bugs = model.bugs.bcmixmeta.bilateral.bias
+  } else {
+    model.bugs.connection = textConnection(model.bugs.bcmixmeta.delta)
+    model.bugs = model.bugs.bcmixmeta.delta
+  }
 
 
-  if(!is.null(x)) {model.bugs.connection = textConnection(model.bugs.bcmixmeta.x)}
+  if(!is.null(x)) {
+    model.bugs.connection = textConnection(model.bugs.bcmixmeta.x)
+    model.bugs = model.bugs.bcmixmeta.x
+  }
 
 
   # Use R2jags as interface for JAGS ...
 
-  results <- jags( data = data.bcmixmeta,
-                   parameters.to.save = par.bcmixmeta,
-                   model.file = model.bugs.connection,
-                   n.chains = nr.chains,
-                   n.iter = nr.iterations,
-                   n.burnin = nr.burnin,
-                   n.thin = nr.thin,
-                   pD = TRUE)
+  if (is.null(parallel)) { #execute R2jags
+    model.bugs.connection <- textConnection(model.bugs)
+
+    # Use R2jags as interface for JAGS ...
+
+    results <- jags( data = data.bcmixmeta,
+                     parameters.to.save = par.bcmixmeta,
+                     model.file = model.bugs.connection,
+                     n.chains = nr.chains,
+                     n.iter = nr.iterations,
+                     n.burnin = nr.burnin,
+                     n.thin = nr.thin,
+                     DIC = TRUE,
+                     pD=TRUE)
+
+    # Close text connection
+    close(model.bugs.connection)
+  }else if(parallel == "jags.parallel"){
+    writeLines(model.bugs, "model.bugs")
+    results <- jags.parallel(     data = data.bcmixmeta,
+                                  parameters.to.save = par.bcmixmeta,
+                                  model.file = "model.bugs",
+                                  n.chains = nr.chains,
+                                  n.iter = nr.iterations,
+                                  n.burnin = nr.burnin,
+                                  n.thin = nr.thin,
+                                  DIC=TRUE)
+
+    #Compute pD from result
+    results$BUGSoutput$pD = results$BUGSoutput$DIC - results$BUGSoutput$mean$deviance
+
+    # Delete model.bugs on exit ...
+    unlink("model.bugs")
+  }
 
-  # Close text connection
-  close(model.bugs.connection)
 
   # Extra outputs that are linked with other functions ...
   results$data = data
