@@ -24,14 +24,15 @@
 #' @param alpha.1             Upper bound of the uniform prior for the concentration parameter for the DPM,
 #'                            default value is alpha.1 = 10.
 #'
-#' @param K                   Maximum number of clusters in the DPM, default value is K = 30.
+#' @param K                   Maximum number of clusters in the DP, default value is K = 30.
 #'
 #'
 #' @param nr.chains           Number of chains for the MCMC computations, default 2.
 #' @param nr.iterations       Number of iterations after adapting the MCMC, default is 10000. Some models may need more iterations.
 #' @param nr.adapt            Number of iterations in the adaptation process, default is 1000. Some models may need more iterations during adptation.
 #' @param nr.burnin           Number of iteration discard for burn-in period, default is 1000. Some models may need a longer burnin period.
-#' @param nr.thin             Thinning rate, it must be a positive integer, the default value 1.
+#' @param nr.thin             Thinning rate, it must be a positive integer, the default value 1
+#' @param parallel            NULL -> jags, 'jags.parallel' -> jags.parallel execution
 #'
 #' @return                    This function returns an object of the class "dpmeta". This object contains the MCMC
 #'                            output of each parameter and hyper-parameter in the model and
@@ -88,7 +89,8 @@ dpmeta = function(
     nr.iterations   = 10000,
     nr.adapt        = 1000,
     nr.burnin       = 1000,
-    nr.thin         = 1)UseMethod("dpmeta")
+    nr.thin         = 1,
+    parallel        = NULL)UseMethod("dpmeta")
 
 
 #' @export
@@ -101,15 +103,16 @@ dpmeta.default = function(
     df.scale.between    = 1,
     alpha.0             = 0.03,
     alpha.1             = 10,
-    K                   = 5,
+    K                   = 30,
     # MCMC setup........................................................
     nr.chains       = 2,
     nr.iterations   = 10000,
     nr.adapt        = 1000,
     nr.burnin       = 1000,
-    nr.thin         = 1)
+    nr.thin         = 1,
+    parallel        = NULL)
 {
-
+  if(!is.null(parallel) && parallel != "jags.parallel") stop("The parallel option must be NULL or 'jags.parallel'")
   # Data
   y = data$TE
   se.y = data$seTE
@@ -121,14 +124,19 @@ dpmeta.default = function(
 
 
   # This list describes the data used by the BUGS script.
-  data.dp = list ("y", "se.y", "N",
-                   "mean.mu.0",
-                   "sd.mu.0",
-                   "scale.sigma.between",
-                   "df.scale.between",
-                   "alpha.0",
-                   "alpha.1",
-                   "K")
+  data.dp <-
+    list(y = y,
+         se.y = se.y,
+         N = N,
+         mean.mu.0 = mean.mu.0,
+         sd.mu.0 = sd.mu.0,
+         scale.sigma.between = scale.sigma.between,
+         df.scale.between = df.scale.between,
+         alpha.0 = alpha.0,
+         alpha.1 = alpha.1,
+         K = K
+    )
+
 
   # List of parameters
   par.dp  <- c("mu.k",
@@ -190,21 +198,41 @@ dpmeta.default = function(
   }
   "
 
-  model.bugs.connection <- textConnection(model.bugs)
+  if (is.null(parallel)) { #execute R2jags
+    model.bugs.connection <- textConnection(model.bugs)
 
-  # Use R2jags as interface for JAGS ...
+    # Use R2jags as interface for JAGS ...
 
-  results <- jags( data = data.dp,
-                   parameters.to.save = par.dp,
-                   model.file = model.bugs.connection,
-                   n.chains = nr.chains,
-                   n.iter = nr.iterations,
-                   n.burnin = nr.burnin,
-                   n.thin = nr.thin,
-                   pD = TRUE)
+    results <- jags( data = data.dp,
+                     parameters.to.save = par.dp,
+                     model.file = model.bugs.connection,
+                     n.chains = nr.chains,
+                     n.iter = nr.iterations,
+                     n.burnin = nr.burnin,
+                     n.thin = nr.thin,
+                     DIC = TRUE,
+                     pD=TRUE)
+
+    # Close text connection
+    close(model.bugs.connection)
+  }else if(parallel == "jags.parallel"){
+    writeLines(model.bugs, "model.bugs")
+    results <- jags.parallel(     data = data.dp,
+                                  parameters.to.save = par.dp,
+                                  model.file = "model.bugs",
+                                  n.chains = nr.chains,
+                                  n.iter = nr.iterations,
+                                  n.burnin = nr.burnin,
+                                  n.thin = nr.thin,
+                                  DIC=TRUE)
+
+    #Compute pD from result
+    results$BUGSoutput$pD = results$BUGSoutput$DIC - results$BUGSoutput$mean$deviance
+
+    # Delete model.bugs on exit ...
+    unlink("model.bugs")
+  }
 
-  # Close text connection
-  close(model.bugs.connection)
 
   # Extra outputs that are linked with other functions ...
   results$data = data
